﻿// Ratio column to show ratio as 2 integers separated by a colon
// image ratios are defined as 'width:height' of image
// written as a colon separating two integers
/*
can put your columns into the main categories now,
Don't think it's fully documented yet,
but with script columns it's set via ScriptColumn.category,
which takes one of these string values:

"loc"
"date"
"size"
"std"
"dims"
"image"
"music"
"prog"
"doc"
"movie"
"sums"
"script"
"shell"
"other"
"eval"
*/

function OnInit(w) 
{
    w.name              = 'Ratio';
    w.group             = "Columns";
    w.version           = '1.0';
    w.desc              = 'Column to show aspect ratio of image as 2 integers seperated by a colon';
    w.url               = 'https://resource.dopus.com/t/ratio-column/45658/2';
    w.copyright         = '(c)2023';
    w.default_enable    = true;
    w.min_version       = '12.28';
}


function OnAddColumns(z) 
{
    var y           = z.AddColumn();
    y.name          = 'ratiozxy';
    y.label         = '- Ratio';
    y.header        = '- Ratio -';  
    y.category      = "dims";
    y.justify       = 'center';
    y.type          = 'wordsort';
    y.autogroup     = true;
    y.autorefresh   = true;
    y.method        = 'OnRatio';
}


function gcd_subractive(a, b) 
{
    while (a !== b) 
    {
        if (a > b) 
            a = a - b;
        else
            b = b - a;
    }
    return a;
}


function gcd_iterative(x, y) 
{
  while(y) 
  {
    var t = y;
    y = x % y;
    x = t;
  }
  return x;
}


function gcd_recursive(x, y) 
{
  if (y) 
  {
      return gcd_recursive(y, x % y);
  } 
  else 
  {
      return x;
  }
}


function simplifyFraction(numerator, denominator) 
{
  // Get the greatest common factor of the numerator and denominator.
  var gcd = gcd_subractive(numerator, denominator);

  // Divide the numerator and denominator by the GCD.
  return { numerator: numerator / gcd, denominator: denominator / gcd };
}


function OnRatio(x) 
{
    var c = x.item;

    if (c.metadata != 'image')
    {
        x.value = "---";
        return;
    }

    try
    {
        var width = c.metadata.image.picwidth;
        var height = c.metadata.image.picheight;
        var result = simplifyFraction(width, height);
        var antecedent = result.numerator;
        var consequent = result.denominator;

        x.value = antecedent + ":" + consequent;
    }
    catch (error)
    {
        x.value = "---";
    }
}


